==> shinybroker-0.6.20.tar.gz: shinybroker-0.6.20/src/shinybroker/ib_fetch_functions.py <==
import atexit, datetime

from shinybroker.connection import (
    create_ibkr_socket_conn, send_ib_message, read_ib_msg
)
from shinybroker.functionary import functionary
from shinybroker.msgs_to_ibkr import req_sec_def_opt_params
from shinybroker.format_ibkr_inputs import *


def fetch_sec_def_opt_params(
        underlyingConId: int,
        underlyingSymbol: str,
        underlyingSecType: str,
        host='127.0.0.1',
        port=7497,
        client_id=9999,
        futFopExchange="",
        timeout=3
):
    """
    Creates a temporary IBKR client socket at the specified `host`, `port`, and
    `client_id`, then makes a query for the security-defined option parameters
    for the security defined by `underlyingConId`, `underlyingSymbol`,
    `underlyingSecType`, and `futFopExchange`.

    `fetch_sec_def_opt_params` will collect all
    `SECURITY_DEFINITION_OPTION_PARAMETER` messages it receives and return
    them in a dataframe when it receives a
    `SECURITY_DEFINITION_OPTION_PARAMETER_END` message.

    If `timeout` number of seconds elapse before receiving
    `SECURITY_DEFINITION_OPTION_PARAMETER_END`, then `fetch_sec_def_opt_params`
    returns `None`.

    Upon completion, `fetch_sec_def_opt_params` closes the socket it opened.

    Parameters
    ------------
    underlyingConId: int
        `conId` of the underlying security
    underlyingSymbol: str
        Symbol of the underlying security for which you want option parameters.
    underlyingSecType: str
        Type of the underlying security; e.g., "`STK`"
    host: '127.0.0.1'
        Address of a running IBKR client (such as TWS or IBG) that has been
        configured to accept API connections
    port: 7497
        Port of a running IBKR client
    client_id: 9999
        Client ID you want to use for the request. If you are connecting to a
        system that is used by multiple users, then you may wish to set aside an
         ID for this purpose; if you're the only one using the account then
         you probably don't have to worry about it -- just use the default.
    futFopExchange: ""
        Only set this parameter if the underlying is a futures contract; in
        other words, don't change it from the default `""` if your underlying is
         a stock. If your underlying **is** a futures contract, then use
         `futFopExchange` to specify the exchange for which you want option
         parameters. You may still pass in `""` if you want the results to
         include **all** of the exchanges available at IBKR that trade
         options on your specified underlying.

    Examples
    --------
    ```
    {{< include ../examples/fetch_sec_def_opt_params.py >}}
    ```
    """
    ib_conn = create_ibkr_socket_conn(
        host=host, port=port, client_id=client_id
    )
    ib_socket = ib_conn['ib_socket']
    ib_socket.settimeout(1)
    atexit.register(ib_socket.close)

    incoming_msg = read_ib_msg(ib_socket)

    send_ib_message(
        s=ib_socket,
        msg=req_sec_def_opt_params(
            reqId=ib_conn['NEXT_VALID_ID'],
            underlyingSymbol=underlyingSymbol,
            futFopExchange=futFopExchange,
            underlyingSecType=underlyingSecType,
            underlyingConId=underlyingConId
        )
    )

    sdops = []
    start_time = datetime.datetime.now()
    while incoming_msg[0] != functionary['incoming_msg_codes'][
        'SECURITY_DEFINITION_OPTION_PARAMETER_END'
    ]:
        incoming_msg = read_ib_msg(sock=ib_socket)
        if incoming_msg[0] == functionary['incoming_msg_codes'][
            'SECURITY_DEFINITION_OPTION_PARAMETER'
        ]:
            sdops.append(
                format_sec_def_opt_params_input(sdop=incoming_msg[2:]))
        if (datetime.datetime.now() - start_time).seconds > timeout:
            break

    try:
        sec_def_opt_params = pd.concat(
            sdops,
            ignore_index=True
        ).sort_values('exchange')
    except ValueError:
        sec_def_opt_params = None

    return sec_def_opt_params

==> shinybroker-0.6.21.tar.gz: shinybroker-0.6.21/src/shinybroker/ib_fetch_functions.py <==
import atexit

from datetime import datetime
from shinybroker.connection import (
    create_ibkr_socket_conn, send_ib_message, read_ib_msg
)
from shinybroker.format_ibkr_inputs import *
from shinybroker.functionary import functionary
from shinybroker.msgs_to_ibkr import (
    req_historical_data,
    req_sec_def_opt_params
)
from shinybroker.obj_defs import Contract


def fetch_sec_def_opt_params(
        underlyingConId: int,
        underlyingSymbol: str,
        underlyingSecType: str,
        host='127.0.0.1',
        port=7497,
        client_id=9999,
        futFopExchange="",
        timeout=3
):
    """
    Creates a temporary IBKR client socket at the specified `host`, `port`, and
    `client_id`, then makes a query for the security-defined option parameters
    for the security defined by `underlyingConId`, `underlyingSymbol`,
    `underlyingSecType`, and `futFopExchange`.

    `fetch_sec_def_opt_params` will collect all
    `SECURITY_DEFINITION_OPTION_PARAMETER` messages it receives and return
    them in a dataframe when it receives a
    `SECURITY_DEFINITION_OPTION_PARAMETER_END` message.

    If `timeout` number of seconds elapse before receiving
    `SECURITY_DEFINITION_OPTION_PARAMETER_END`, then `fetch_sec_def_opt_params`
    returns `None`.

    Upon completion, `fetch_sec_def_opt_params` closes the socket it opened.

    Parameters
    ------------
    underlyingConId: int
        `conId` of the underlying security
    underlyingSymbol: str
        Symbol of the underlying security for which you want option parameters.
    underlyingSecType: str
        Type of the underlying security; e.g., "`STK`"
    host: '127.0.0.1'
        Address of a running IBKR client (such as TWS or IBG) that has been
        configured to accept API connections
    port: 7497
        Port of a running IBKR client
    client_id: 9999
        Client ID you want to use for the request. If you are connecting to a
        system that is used by multiple users, then you may wish to set aside an
         ID for this purpose; if you're the only one using the account then
         you probably don't have to worry about it -- just use the default.
    futFopExchange: ""
        Only set this parameter if the underlying is a futures contract; in
        other words, don't change it from the default `""` if your underlying is
         a stock. If your underlying **is** a futures contract, then use
         `futFopExchange` to specify the exchange for which you want option
         parameters. You may still pass in `""` if you want the results to
         include **all** of the exchanges available at IBKR that trade
         options on your specified underlying.

    Examples
    --------
    ```
    {{< include ../examples/fetch_sec_def_opt_params.py >}}
    ```
    """
    ib_conn = create_ibkr_socket_conn(
        host=host, port=port, client_id=client_id
    )
    ib_socket = ib_conn['ib_socket']
    ib_socket.settimeout(1)

    incoming_msg = read_ib_msg(ib_socket)

    send_ib_message(
        s=ib_socket,
        msg=req_sec_def_opt_params(
            reqId=ib_conn['NEXT_VALID_ID'],
            underlyingSymbol=underlyingSymbol,
            futFopExchange=futFopExchange,
            underlyingSecType=underlyingSecType,
            underlyingConId=underlyingConId
        )
    )

    sdops = []
    start_time = datetime.now()
    while incoming_msg[0] != functionary['incoming_msg_codes'][
        'SECURITY_DEFINITION_OPTION_PARAMETER_END'
    ]:
        incoming_msg = read_ib_msg(sock=ib_socket)
        if incoming_msg[0] == functionary['incoming_msg_codes'][
            'SECURITY_DEFINITION_OPTION_PARAMETER'
        ]:
            sdops.append(
                format_sec_def_opt_params_input(sdop=incoming_msg[2:]))
        if (datetime.now() - start_time).seconds > timeout:
            break

    try:
        sec_def_opt_params = pd.concat(
            sdops,
            ignore_index=True
        ).sort_values('exchange')
    except ValueError:
        sec_def_opt_params = None

    ib_socket.close()

    return sec_def_opt_params


def fetch_historical_data(
        contract: Contract,
        endDateTime="",
        durationStr="1 D",
        barSizeSetting='1 hour',
        whatToShow='Trades',
        useRTH=True,
        host='127.0.0.1',
        port=7497,
        client_id=9999,
        timeout=3
):
    """
    Creates a temporary IBKR client socket at the specified `host`, `port`, and
    `client_id`, then makes a query for the historical data specified by the
    input parameters, receives the response, closes the socket, formats the
    response, and returns the result.

    If `timeout` number of seconds elapse before receiving historical data,
    then `fetch_historical_data` returns `None`.

    Parameters
    ------------
    underlyingConId: int
        `conId` of the underlying security
    underlyingSymbol: str
        Symbol of the underlying security for which you want option parameters.
    underlyingSecType: str
        Type of the underlying security; e.g., "`STK`"
    host: '127.0.0.1'
        Address of a running IBKR client (such as TWS or IBG) that has been
        configured to accept API connections
    port: 7497
        Port of a running IBKR client
    client_id: 9999
        Client ID you want to use for the request. If you are connecting to a
        system that is used by multiple users, then you may wish to set aside an
         ID for this purpose; if you're the only one using the account then
         you probably don't have to worry about it -- just use the default.

    Examples
    --------
    ```
    {{< include ../examples/fetch_historical_data.py >}}
    ```
    """

    # contract = Contract({
    #     'symbol': "AAPL",
    #     'secType': "STK",
    #     'exchange': "SMART",
    #     'currency': "USD"
    # })
    # endDateTime = ""
    # durationStr = "1 D"
    # barSizeSetting = '1 hour'
    # whatToShow = 'Trades'
    # useRTH = True
    # host = '127.0.0.1'
    # port = 7497
    # client_id = 9999
    # timeout = 3

    ib_conn = create_ibkr_socket_conn(
        host=host, port=port, client_id=client_id
    )
    ib_socket = ib_conn['ib_socket']
    atexit.register(ib_socket.close)

    incoming_msg = read_ib_msg(ib_socket)

    send_ib_message(
        s=ib_socket,
        msg=req_historical_data(
            reqId=1,
            contract=contract,
            endDateTime=endDateTime,
            durationStr=durationStr,
            barSizeSetting=barSizeSetting,
            whatToShow=whatToShow,
            useRTH=useRTH,
            formatDate=2,
            keepUpToDate=False
        )
    )

    historical_data = None

    while True:
        incoming_msg = read_ib_msg(sock=ib_socket)
        if incoming_msg[0] == '4' and incoming_msg[3] == '321':
            print(incoming_msg[4])
            break
        if incoming_msg[0] == functionary['incoming_msg_codes'][
            'HISTORICAL_DATA'
        ]:
            historical_data = format_historical_data_input(incoming_msg[1:])
            break

    ib_socket.close()

    return historical_data
